; ==============================================
; Wonderland Custom Content Enabler
; ---------------------------------------------
; Originally made by Matthieu Parizeau, decompiled and reconstructed by Lazy Loof
; Extracts files packed inside res1.dat and res2.dat
; ================================================


; CONSTANTS
; -------------------
Const NULL_PTR = 0 ; used as syntatic sugar to distinguish between normal zeros and Blitz3D objects


; MAIN PROGRAM
; -------------------
Graphics 800, 600, 32, 2
SetBuffer BackBuffer()

AppTitle "Wonderland Extracter"

PrintHeader()
ExtractAllFiles()
PrintSeparator()
Print "Press any key to exit..."
WaitKey()
End


; FUNCTIONS
; --------------------

; extract all files from packed file
Function ExtractAllFiles()
	; get res2.dat first
	Local Res2$ = FindRes2()
	If Res2 = "nofile" Then
		Color 255,0,0
		Print "res#.dat that is greater than 5MB not found!"
		Color 255,255,255
		Return False
	End If
	; check if custom content already enabled
	If FileSize("res1.dat") = 0 Or FileSize(Res2) = 0 Then
		Color 255, 0, 0
		Print "Custom Content Already Enabled...Skipping"
		Color 255, 255, 255
		Return False
	End If
	; now load files
	res1_file_handle = ReadFile("res1.dat")
	If res1_file_handle = NULL_PTR Then
		Color 255, 0, 0
		Print "res1.dat not found!"
		Color 255, 255, 255
	End If
	res2_file_handle = ReadFile(Res2)
	If res2_file_handle = NULL_PTR Then
		Color 255, 0, 0
		Print "res2.dat not found!"
		Color 255, 255, 255
	End If
	If res1_file_handle = NULL_PTR Or res2_file_handle = NULL_PTR Then
		Return False
	End If
	; EXTRACT!
	PrintHead("<---Extracting Resfiles--->")
	Repeat
		PackFileName$ = ReadString(res1_file_handle)
		If Len(PackFileName)=0 Then
			; no name, must be a corrupted res1.dat
			; exit
			Exit
		End If
		PackFilePosition = ReadInt(res1_file_handle)
		PackFileSize = ReadInt(res1_file_handle)
		Print "Extracting File: '" + PackFileName + "'"
		; make up a memory bank to hold the file contents!
		file_bank_handle = CreateBank(PackFileSize)
		ReadBytes file_bank_handle, res2_file_handle, 0, PackFileSize
		; create the parent folders first
		TryCreateFolder(PackFileName)
		; now write the contents of bank into the file
		dest_file_handle = WriteFile(PackFileName)
		WriteBytes file_bank_handle, dest_file_handle, 0, PackFileSize
		; done, now free up stuff
		CloseFile(dest_file_handle)
		FreeBank file_bank_handle
	Forever
	; all set, now close the files
	CloseFile(res1_file_handle)
	CloseFile(res2_file_handle)
	; blank out the res# files!
	PrintHead("<---Blanking Resfiles--->")
	BlankFile("res1.dat")
	BlankFile(Res2)
End Function

; blank the packed files
Function BlankFile(file_name$)
	Del(file_name)
	CreateEmptyFile(file_name)
End Function

; create an empty file
Function CreateEmptyFile(file_name$)	
	Print "Creating File: " + file_name
	file_handle = WriteFile(file_name)
	CloseFile(file_handle)
End Function

; delete a file and log
Function Del(file_name$)
	Print "Deleting File: " + file_name
	DeleteFile file_name
End Function

; create the directory along with all the parent directories if neccessary
; note from Lazy Loof: looks like a bad algorithm, should be changed with one used in OpenWA
; (check "MyCreateDir" function "shared/util.bb" in OpenWA)
Function TryCreateFolder(folder_name$)
	Local result$ = ""
	Local index = 0
	Repeat
		index = index + 1
		Select Instr(folder_name, "\", Len(folder_name) - index)
			Case 0
				If index >= Len(folder_name) Then
					result = "Error"
					Exit
				End If
			Default
				result = Left(folder_name, (Len(folder_name) - index) - 1)
				Exit
		End Select
	Forever
	CreateDir result
	If FileType(result) = 0 Then
		TryCreateFolder(result)
		TryCreateFolder(folder_name)
	End If
End Function

; find proper res#.dat file
Function FindRes2$()
	Local dir_handle = ReadDir(".")
	Local next_file$ = ""
	Repeat
		next_file = NextFile(dir_handle)
		If next_file = "" Then
			Exit
		End If
		If Right(next_file, 4) = ".dat" And Left(next_file, 3) = "res" And FileSize(next_file)>5242880 Then
			CloseDir dir_handle
			Return next_file
		End If
	Forever
	CloseDir dir_handle
	Return "nofile"
End Function

; print header
Function PrintHeader()
	Color 0,255,255
	Print "========Wonderland Custom Content Enabler========"
	Print "===========Created by Matthieu Parizeau=========="
	Color 255,255,255
	PrintSeparator()
End Function

; print separator line
Function PrintSeparator()	
	Color 255,255,0
	Print "-------------------------------------------------"
	Print ""
	Color 255,255,255
End Function

; print a text as head
Function PrintHead(headText$)	
	Color 0,255,0
	Print headText
	Color 255,255,255
	PrintSeparator()
End Function

; ==============================================
; END OF FILE
; ==============================================